@set _api_version=Huawei web-api v0.4 (c) Andronidus for 4pda.ru

@echo off & goto _apiParseCommands

:: ========================================
:apiInit
:: ========================================
:: apiInit [<ip>] -- init variables and set web-ip
:: result errorlevel:
:: 0 - no errors
:: 1 - curl not found
:: 2 - xml parser not found
:: 3 - ip not reachable (no web-ui)
:: 4 - %api_result% create error

:: ------------------------------
:: default settings
:: web-ip
if not defined api_ip set api_ip=192.168.8.1

:: connection timeout
if not defined api_timeout set api_timeout=3

:: file name to which the curl response will be written
if not defined api_result set api_result="_api_result"

:: path to curl.exe
if not defined curl set curl=bin\curl\curl.exe

:: path to xmlstartlet
if not defined xml set xml=bin\xmlstarlet-1.6.1\xml.exe

:: const of xml header -- u may use it for data requests
set "api_xml=<?xml version='1.0' encoding='UTF-8'?>"

:: ------------------------------
:: work variables
if not "%~1"=="" set api_ip=%~1

:: add current time and random number as suffix of result file
set _t=%time: =%& call set _t=%%_t::=^%%& call set _t=%%_t:,=^%%
set api_result="%api_result:"=%.%_t%-%random%"

set api_sesID=
set api_tokID=
set api_http_error=0
set api_error_code=
set api_error_msg=
set _api_curl_to=--connect-timeout %api_timeout%
set _api_curl_auth=
set _api_valroot=
if not defined _api_echo set _api_echo=
if not defined _api_debug set _api_debug=0
call :apiDebug %_api_debug%

:: ------------------------------
:: check errors
call :_apiFindFile %curl% || exit /b 1
call :_apiFindFile %xml% || exit /b 2
%curl% -s %api_ip%:80 --connect-timeout 1 > nul || exit /b 3
echo 1 >%api_result% || exit /b 4
del %api_result%

exit /b 0

:: ========================================
:_apiFindFile
:: ========================================
:: find file localy and in PATH environment
:: return: 0 - if found, 1 - not found
if exist "%~1" exit /b 0
setlocal
for %%X in ("%~1") do (set found=%%~$PATH:X)
if defined found (set found=0) else (set found=1)
endlocal & exit /b %found%

:: ========================================
:_apiParseCommands
:: ========================================
:: parse command-line commands
:: return: command result OR -1 if unknown command

:: check for valid commands
set api_cmd=%1
if /i "%~1"=="Init"     goto _apiCmd
if /i "%~1"=="Auth"     goto _apiCmd
if /i "%~1"=="Get"      goto _apiCmd
if /i "%~1"=="Post"     goto _apiCmd
if /i "%~1"=="Req"      goto _apiCmd
if /i "%~1"=="GetError" goto _apiCmd
if /i "%~1"=="Val"      goto _apiCmd
if /i "%~1"=="ValRoot"  goto _apiCmd
if /i "%~1"=="Debug"    goto _apiCmd
if /i "%~1"=="Echo"     goto _apiCmd
if /i "%~1"=="Done"     goto _apiCmd

:: unknown command
set _api_ret=-1
if not "%~1"=="" echo Unknown hw-api command: %~1 & goto _apiCmdDone

cls
echo %_api_version%
echo.
echo see manual and examples for use
echo.
goto _apiCmdDone

:_apiCmd
:: call api command function with given parameters
call :api%~1 %2 %3 %4
set _api_ret=%errorlevel%

:_apiCmdDone
set api_error=%_api_ret%
if %_api_ret%==0 set api_error=

if defined _api_echo @echo on
@exit /b %_api_ret%

:: ========================================
:apiAuth
:: ========================================
:: request and parse sessionID and tokenID
:: return: 0 - no errors; 10 - no ID; other codes - see apiReq

set api_sesID=
set api_tokID=
set _api_curl_auth=

call :apiGet api/webserver/SesTokInfo || exit /b
call :apiVal "/response/SesInfo" api_sesID
call :apiVal "/response/TokInfo" api_tokID

if defined api_sesID (
	if defined api_tokID set _api_curl_auth=-H "Cookie: %api_sesID%" -H "__RequestVerificationToken: %api_tokID%"
	exit /b 0
)

set api_sesID=
set api_tokID=
exit /b 10

:: ========================================
:apiGet
:: ========================================
:: apiGet <url-api-path> [<data>] -- send GET request with or w/o data
:: result: apiReq result

call :apiReq GET "%~1" %2
exit /b %errorlevel%

:: ========================================
:apiPost
:: ========================================
:: apiPost <url-api-path> [<data>] -- send POST request with or w/o data
:: result: apiReq result

call :apiReq POST "%~1" %2
exit /b %errorlevel%

:: ========================================
:apiReq
:: ========================================
:: apiReq <req-mode> <url-path> [<data>] -- send GET or POST request (req-mode) with or w/o data
:: result:
:: 0 - no errors (equals http response 200)
:: < 0 - curl error codes
:: 1 - HTTP error code (see api_http_error)
:: 2 - web-api error response (see api_error_code)

if exist %api_result% del %api_result%
call :apiValRoot

:: check for data
set _d=%3
if defined _d set _d=-d %_d%

:: skip leading "/" in url
set _u=%~2
if "%_u:~,1%"=="/" set _u=%_u:~1%

@if %_api_debug% GTR 0 @echo on
for /f %%E in ('%curl% -w "%%{http_code}" %_api_curl_debug% %_api_curl_to% %_api_curl_auth% -X %1 %_d% "http://%api_ip%/%_u%" -o %api_result% ^& call echo :%%^^errorlevel%%') do @(set _api_ret=%%E)
@if %_api_debug% GTR 0 @echo off

:: split errorlevel and HTTP response code
set api_http_error=%_api_ret:~,3%
set _api_ret=%_api_ret:~4%

:: curl errors
if not %_api_ret%==0 (
	set api_http_error=
	exit /b -%_api_ret%
)

:: http errors
if not %api_http_error%==200 exit /b 1

:: api error code
call :apiGetError || exit /b 2

:: no errors
exit /b 0

:: ========================================
:apiGetError
:: ========================================
:: get web-api error code
:: return: 0 - if no errors, not 0 - error code
call :apiVal "/error/code" api_error_code
call :apiVal "/error/message" api_error_msg

if not defined api_error_code exit /b 0
exit /b %api_error_code%

:: ========================================
:apiValRoot
:: ========================================
:: apiValRoot [<root>] -- set root node (XPath), after that u can get values without full path
set _api_valroot=
if not "%~1"=="" set _api_valroot="%~1"
exit /b 0

:: ========================================
:apiVal
:: ========================================
:: apiVal <XPath> <var-name> -- get value from xml result (server response) and store it in variable <var-name>
:: return: 0 - on success, 1- if no api_result 
if not exist %api_result% (
	set "%~2="
	exit /b 1
)
set _m=
set _api_ret=
if defined _api_valroot set _m=-m %_api_valroot%
for /f "delims=" %%V in ('%xml% sel -t %_m% -v "%~1" %api_result%') do set "%~2=%%V"
exit /b 0

:: ========================================
:apiDebug
:: ========================================
:: apiDebug [<0|1|2>] -- show some debug info (curl requests): 0 - disable, 1 - short info, 2 - verbose info
:: result: debug status (0,1,2)

if "%~1"=="" goto __apiDebug

set _api_curl_debug=-s

if "%~1"=="0" set _api_debug=0 & goto __apiDebug

if "%~1"=="2" (
	set _api_debug=2
	set _api_curl_debug=-v
) else (
	set _api_curl_debug=
	set _api_debug=1
)

:__apiDebug
exit /b %_api_debug%

:: ========================================
:apiEcho
:: ========================================
:: apiEcho [<0|1>] -- switch off (0) or switch on (1) echo on exist
:: result: echo status (0 or 1)

if not "%~1"=="" (
	set _api_echo=
	if not "%~1"=="0" set _api_echo=1
)
if defined _api_echo exit /b 1
exit /b 0

:: ========================================
:apiDone
:: ========================================
if exist %api_result% del %api_result%
exit /b 0

